@@ -3,6 +3,7 @@ import sys
 import unittest
 
 from peewee import *
+from wtforms import __version__ as wtforms_version
 from wtforms import fields as wtfields
 from wtforms.form import Form as WTForm
 from wtforms.validators import Regexp
@@ -37,7 +38,7 @@ class Blog(TestModel):
 
 @implements_to_string
 class Entry(TestModel):
-    pk = PrimaryKeyField()
+    pk = AutoField()
     blog = ForeignKeyField(Blog)
     title = CharField(verbose_name='Wacky title')
     content = TextField()
@@ -63,6 +64,11 @@ class ChoicesModel(TestModel):
     true_or_false = BooleanField(choices=((True, 't'), (False, 'f')))
 
 
+class BlankChoices(TestModel):
+    status = IntegerField(choices=((1, 'One'), (2, 'Two')),
+                          null=True)
+
+
 class NonIntPKModel(TestModel):
     id = CharField(primary_key=True)
     value = CharField()
@@ -72,6 +78,7 @@ BlogForm = model_form(Blog)
 EntryForm = model_form(Entry)
 NullFieldsModelForm = model_form(NullFieldsModel)
 ChoicesForm = model_form(ChoicesModel, field_args={'salutation': {'choices': (('mr', 'Mr.'), ('mrs', 'Mrs.'))}})
+BlankChoicesForm = model_form(BlankChoices)
 NonIntPKForm = model_form(NonIntPKModel, allow_pk=True)
 
 class FakePost(dict):
@@ -187,7 +194,11 @@ class WTFPeeweeTestCase(unittest.TestCase):
         choices_obj.gender = 'x'
         form = ChoicesForm(obj=choices_obj)
         self.assertFalse(form.validate())
-        self.assertEqual(form.errors, {'gender': ['Not a valid choice']})
+        if wtforms_version[0] == '2':
+            errmsg = 'Not a valid choice'
+        else:
+            errmsg = 'Not a valid choice.'
+        self.assertEqual(form.errors, {'gender': [errmsg]})
 
         choices_obj.gender = 'm'
         choices_obj.status = '1'
@@ -195,20 +206,57 @@ class WTFPeeweeTestCase(unittest.TestCase):
         self.assertEqual(form.status.data, 1)
         self.assertTrue(form.validate())
 
-        choices_obj.status = '3'
-        form = ChoicesForm(obj=choices_obj)
+        # "3" is not a valid status.
+        form = ChoicesForm(FakePost({'status': '3'}), obj=choices_obj)
 
+        # Invalid choice -- must be 1 or 2.
         self.assertFalse(form.validate())
+        self.assertTrue(list(form.errors), ['status'])
 
+        # Nullable field with choices:
         choices_obj.status = None
         form = ChoicesForm(obj=choices_obj)
         self.assertEqual(form.status.data, None)
         self.assertTrue(form.validate())
+        self.assertFalse(list(form.errors), ['status'])
+
+        # Not-nullable field with choices:
+        choices_obj.gender = None
+        form = ChoicesForm(obj=choices_obj)
+        self.assertEqual(form.status.data, None)
+        self.assertFalse(form.validate())
+        self.assertTrue(list(form.errors), ['gender'])
+
+    def test_blank_choices(self):
+        obj = BlankChoices(status=None)
+        form = BlankChoicesForm(obj=obj)
+        self.assertTrue(form.validate())
+
+        # Ensure that the "None" status value is set when populating an object
+        # (overwriting a previous non-empty value).
+        new_obj = BlankChoices(status=1)
+        form.populate_obj(new_obj)
+        self.assertTrue(new_obj.status is None)
+
+        new_obj = BlankChoices(status=1)
+        form = BlankChoicesForm(FakePost({'status': ''}))
+        self.assertTrue(form.validate())
+        form.populate_obj(new_obj)
+        self.assertTrue(new_obj.status is None)
+
+        new_obj = BlankChoices()
+        form = BlankChoicesForm(FakePost({'status': 1}))
+        self.assertTrue(form.validate())
+        form.populate_obj(new_obj)
+        self.assertEqual(new_obj.status, 1)
+
+        form = BlankChoicesForm(FakePost({'status': 3}))
+        self.assertFalse(form.validate())
 
     def test_blog_form(self):
         form = BlogForm()
         self.assertEqual(list(form._fields.keys()), ['title'])
-        self.assertTrue(isinstance(form.title, wtfields.TextField))
+        self.assertTrue(isinstance(form.title, wtfields.StringField))
         self.assertEqual(form.data, {'title': None})
 
     def test_entry_form(self):
@@ -218,7 +266,7 @@ class WTFPeeweeTestCase(unittest.TestCase):
         self.assertTrue(isinstance(form.blog, ModelSelectField))
         self.assertTrue(isinstance(form.content, wtfields.TextAreaField))
         self.assertTrue(isinstance(form.pub_date, WPDateTimeField))
-        self.assertTrue(isinstance(form.title, wtfields.TextField))
+        self.assertTrue(isinstance(form.title, wtfields.StringField))
 
         self.assertEqual(form.title.label.text, 'Wacky title')
         self.assertEqual(form.blog.label.text, 'Blog')
@@ -232,7 +280,7 @@ class WTFPeeweeTestCase(unittest.TestCase):
 
         # check that the options look right
         self.assertEqual(list(form.blog.iter_choices()), [
-            (self.blog_a.get_id(), u'a', False), (self.blog_b.get_id(), u'b', False)
+            (self.blog_a._pk, u'a', False), (self.blog_b._pk, u'b', False)
         ])
 
     def test_blog_form_with_obj(self):
@@ -252,7 +300,7 @@ class WTFPeeweeTestCase(unittest.TestCase):
 
         # check that the options look right
         self.assertEqual(list(form.blog.iter_choices()), [
-            (self.blog_a.get_id(), u'a', True), (self.blog_b.get_id(), u'b', False)
+            (self.blog_a._pk, u'a', True), (self.blog_b._pk, u'b', False)
         ])
 
     def test_blog_form_saving(self):
@@ -294,7 +342,7 @@ class WTFPeeweeTestCase(unittest.TestCase):
             'content': 'some content',
             'pub_date-date': '2011-02-01',
             'pub_date-time': '00:00:00',
-            'blog': self.blog_b.get_id(),
+            'blog': self.blog_b._pk,
         }))
         self.assertTrue(form.validate())
 
@@ -320,7 +368,7 @@ class WTFPeeweeTestCase(unittest.TestCase):
             'title': 'a1 edited',
             'content': 'a1 content',
             'pub_date': '2011-01-01 00:00:00',
-            'blog': self.blog_b.get_id(),
+            'blog': self.blog_b._pk,
         }), obj=self.entry_a1)
         self.assertTrue(form.validate())
 
@@ -417,18 +465,18 @@ class WTFPeeweeTestCase(unittest.TestCase):
     def test_hidden_field(self):
         class TestEntryForm(WTForm):
             blog = HiddenQueryField(query=Blog.select())
-            title = wtfields.TextField()
+            title = wtfields.StringField()
             content = wtfields.TextAreaField()
 
         form = TestEntryForm(FakePost({
             'title': 'new entry',
             'content': 'some content',
-            'blog': self.blog_b.get_id(),
+            'blog': self.blog_b._pk,
         }))
 
         # check the htmlz for the form's hidden field
         html = form._fields['blog']()
-        self.assertEqual(html, u'<input id="blog" name="blog" type="hidden" value="%s">' % self.blog_b.get_id())
+        self.assertEqual(html, u'<input id="blog" name="blog" type="hidden" value="%s">' % self.blog_b._pk)
 
         self.assertTrue(form.validate())
         self.assertEqual(form.blog.data, self.blog_b)
@@ -451,12 +499,12 @@ class WTFPeeweeTestCase(unittest.TestCase):
         form = TestEntryForm(FakePost({
             'title': 'a1 edited',
             'content': 'a1 content',
-            'blog': self.blog_b.get_id(),
+            'blog': self.blog_b._pk,
         }), obj=self.entry_a1)
 
         # check the htmlz for the form's hidden field
         html = form._fields['blog']()
-        self.assertEqual(html, u'<input id="blog" name="blog" type="hidden" value="%s">' % self.blog_b.get_id())
+        self.assertEqual(html, u'<input id="blog" name="blog" type="hidden" value="%s">' % self.blog_b._pk)
 
         self.assertTrue(form.validate())
 
